unit Vars;

interface

uses
  Database,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Buttons, Menus, DB, Wwdatsrc, Grids, Wwdbigrd,
  Wwdbgrid, ExtCtrls, DBTables, Wwtable;

type
  TFormVars = class(TForm)
    BtnDone: TBitBtn;
    BtnHelp: TBitBtn;
    MainMenu1: TMainMenu;
    Main1: TMenuItem;
    MenuHelp: TMenuItem;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    GridVariables: TwwDBGrid;
    EditVariable: TEdit;
    BtnAdd: TBitBtn;
    BtnDelete: TBitBtn;
    DSVariables: TwwDataSource;
    Filter: TLabel;
    EditInitialization: TEdit;
    Label1: TLabel;
    RBClear: TRadioButton;
    Bevel4: TBevel;
    RBSave: TRadioButton;
    RBValue: TRadioButton;
    Bevel5: TBevel;
    EditSet: TEdit;
    Label2: TLabel;
    BtnSet: TBitBtn;
    Bevel7: TBevel;
    Panel1: TPanel;
    Status: TLabel;
    BtnVars: TBitBtn;
    CtrlR1: TMenuItem;
    procedure BtnDoneClick(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure BtnHelpClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
    procedure GridVariablesCalcCellColors(Sender: TObject; Field: TField;
      State: TGridDrawState; highlight: Boolean; AFont: TFont;
      ABrush: TBrush);
    procedure FormShow(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure EditVariableKeyPress(Sender: TObject; var Key: Char);
    procedure EditInitializationKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure GridVariablesDblClick(Sender: TObject);
    procedure GridVariablesKeyPress(Sender: TObject; var Key: Char);
    procedure RBClearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EditSetKeyPress(Sender: TObject; var Key: Char);
    procedure BtnSetClick(Sender: TObject);
    procedure BtnVarsClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    nodelete: boolean;
  end;

var
  FormVars: TFormVars;

implementation

{$R *.DFM}

uses  VarGrabber, Main;

procedure TFormVars.FormCreate(Sender: TObject);
begin
  nodelete:=false;
end;

procedure TFormVars.BtnDoneClick(Sender: TObject);
var table: TwwTable;
begin
  GridVariables.hide;
  table:=TwwTable.create(self);
  with table do begin
    databasename:=Datawell.bde;
    tablename:='Variables.db';
    wwfilter.assign(Datawell.TableVariables.wwfilter);
    filterActivate;
    active:=true;
    first;
    while not EOF do begin
      FormMain.UpdateCanvasVariable('@'+fieldbyname('Variable').asstring,fieldbyname('Value').asstring);
      next;
    end;
  end;
  table.free;
  close;
end;

procedure TFormVars.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  Status.caption:=' '+(sender as TControl).hint;
end;

procedure TFormVars.BtnHelpClick(Sender: TObject);
begin
  Datawell.Help('variables');
end;

procedure TFormVars.BtnDeleteClick(Sender: TObject);
var
  didfind: boolean;
  work: string;
  vars, events, groups: TStringList;
  i: integer;
begin
  if nodelete then begin
    messagedlg('You can only delete variables when this screen is opened from the main screen',
      mtinformation,[mbok],0);
    exit;
  end;
  if Datawell.TableVariables.EOF then exit;
  didfind:=false;
  vars:=TStringList.create;
  groups:=TStringList.create;
  events:=TStringList.create;
  with Datawell.TableEvents do begin
    refresh;
    wwfilter.clear;
    wwfilter.add('Master = '+Datawell.masterindex);
    filterActivate;
    active:=true;
    first;
    while not EOF do begin
      work:=Datawell.TableEvents.fieldbyname('Action').asstring;
      if copy(work,1,1) = '|' then system.delete(work,1,1);
      Datawell.Explode(work,',',vars);
      if vars.indexof(Datawell.TableVariables.fieldbyname('Variable').asstring) >= 0 then begin
        groups.add(Datawell.TableEvents.fieldbyname('Group').asstring);
        events.add(Datawell.TableEvents.fieldbyname('Event').asstring);
      end;
      next;
    end;
  end;
  if events.count > 0 then begin
    work:='You cannot delete variables assigned to an event'+#13+
      'Event(s) that use this variable (Groups in []):';
    for i:=0 to events.count-1 do work:=work+#13+' ['+groups[i]+'] '+events[i];
    messagedlg(work,mtinformation,[mbok],0);
    exit;
  end;
  events.free;
  groups.free;
  vars.free;
  Datawell.TableVariables.delete;
  GridVariables.refresh;
end;

procedure TFormVars.GridVariablesCalcCellColors(Sender: TObject;
  Field: TField; State: TGridDrawState; highlight: Boolean; AFont: TFont;
  ABrush: TBrush);
begin
  if field.displayname = 'Variable' then begin
    afont.color:=Datawell.TableVariables.fieldbyname('FColor').asinteger;
    afont.style:=[fsbold];
    abrush.color:=Datawell.TableVariables.fieldbyname('BColor').asinteger;
  end
  else if field.displayname = 'Initialization' then begin
    if Datawell.TableVariables.fieldbyname('Initialize').asstring = '<clear on login>' then begin
      afont.color:=clgray;
      afont.style:=[fsbold,fsitalic];
      abrush.color:=clwhite;
    end
    else if Datawell.TableVariables.fieldbyname('Initialize').asstring = '<do not initialize>' then begin
      afont.color:=clwhite;
      afont.style:=[fsbold,fsitalic];
      abrush.color:=clgray;
    end
    else begin
      afont.color:=clblack;
      abrush.color:=clwhite;
    end;
  end;
end;

procedure TFormVars.FormShow(Sender: TObject);
begin
  Datawell.TableVariables.refresh;
end;

procedure TFormVars.EditVariableKeyPress(Sender: TObject; var Key: Char);
begin
  if key = #13 then begin
    key:=#0;
    BtnAdd.click;
  end;
end;

procedure TFormVars.EditInitializationKeyPress(Sender: TObject;
  var Key: Char);
begin
  if key = #13 then begin
    key:=#0;
    BtnAdd.click;
  end;
end;

procedure TFormVars.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (key = VK_DELETE) and (activecontrol = GridVariables) then begin
    BtnDelete.click;
    key:=0;
  end;
end;

procedure TFormVars.GridVariablesDblClick(Sender: TObject);
var init: string;
begin
  if Datawell.TableVariables.EOF then exit;
  EditVariable.text:=Datawell.TableVariables.fieldbyname('Variable').asstring;
  init:=Datawell.TableVariables.fieldbyname('Initialize').asstring;
  EditInitialization.text:='';
  if init = '<clear on login>' then RBClear.checked:=true
  else if init = '<do not initialize>' then RBSave.checked:=true
  else begin
    RBValue.checked:=true;
    EditInitialization.text:=init;
  end;
  EditSet.text:=Datawell.TableVariables.fieldbyname('Value').asstring;
  EditSet.selectall;
  EditSet.setfocus;
end;

procedure TFormVars.BtnAddClick(Sender: TObject);
var
  didfind: boolean;
  failure: string;
  i: integer;
  table: TwwTable;
begin
  didfind:=false;
  if EditVariable.text = '' then begin
    messagedlg('You must enter a variable',mtinformation,[mbok],0);
    EditVariable.setfocus;
    exit;
  end;
  if not Datawell.ValidWordNumber(EditVariable.text) then begin
    messagedlg('Variables can contain only alphanumeric characters',
      mtinformation,[mbok],0);
    EditVariable.setfocus;
    exit;
  end;
//Check for variable extensions of this one
  i:=length(EditVariable.text);
  failure:='';
  table:=TwwTable.create(self);
  with table do begin
    databasename:=Datawell.bde;
    tablename:='Variables.db';
    wwfilter.assign(Datawell.TableVariables.wwfilter);
    filterActivate;
    active:=true;
    first;
    while not EOF do begin
      if (copy(fieldbyname('Variable').asstring,1,i) = EditVariable.text) and
        (EditVariable.text <> fieldbyname('Variable').asstring) then begin
        failure:=fieldbyname('Variable').asstring;
        break;
      end;
      next;
    end;
  end;
  table.free;
  if failure <> '' then begin
    messagedlg('New variables cannot be shortened versions of an existing variable'+#13+
      'The existing variable being compared against is: '+failure,mtinformation,[mbok],0);
    EditVariable.setfocus;
    exit;
  end;
//Check for variables shortened of this one
  failure:='';
  table:=TwwTable.create(self);
  with table do begin
    databasename:=Datawell.bde;
    tablename:='Variables.db';
    wwfilter.assign(Datawell.TableVariables.wwfilter);
    filterActivate;
    active:=true;
    first;
    while not EOF do begin
      i:=length(fieldbyname('Variable').asstring);
      if (copy(EditVariable.text,1,i) = fieldbyname('Variable').asstring) and
        (EditVariable.text <> fieldbyname('Variable').asstring) then begin
        failure:=fieldbyname('Variable').asstring;
        break;
      end;
      next;
    end;
  end;
  table.free;
  if failure <> '' then begin
    messagedlg('New variables cannot be lenghtened versions of an existing variable'+#13+
      'The existing variable being compared against is: '+failure,mtinformation,[mbok],0);
    EditVariable.setfocus;
    exit;
  end;
  didfind:=false;
  table:=TwwTable.create(self);
  with table do begin
    databasename:=Datawell.bde;
    tablename:='Variables.db';
    wwfilter.assign(Datawell.TableVariables.wwfilter);
    filterActivate;
    active:=true;
    first;
    while not EOF do begin
      if fieldbyname('Variable').asstring = EditVariable.text then begin
        didfind:=true;
        break;
      end;
      next;
    end;
  end;
  table.free;
  with Datawell.TableVariables do begin
    if didfind then edit
    else append;
    fieldbyname('Master').asinteger:=Datawell.masterinteger;
    fieldbyname('Variable').asstring:=EditVariable.text;
    if RBClear.checked then
      fieldbyname('Initialize').asstring:='<clear on login>'
    else if RBSave.checked then
      fieldbyname('Initialize').asstring:='<do not initialize>'
    else fieldbyname('Initialize').asstring:=EditInitialization.text;
    fieldbyname('FColor').asinteger:=clwhite;
    fieldbyname('BColor').asinteger:=clnavy;
    fieldbyname('Invisible').asboolean:=false;
    post;
  end;
  EditVariable.text:='';
  EditInitialization.text:='';
  EditVariable.setfocus;
  Datawell.TableVariables.refresh;
  GridVariables.refresh;
end;

procedure TFormVars.GridVariablesKeyPress(Sender: TObject; var Key: Char);
begin
  if key = #13 then begin
    key:=#0;
    GridVariablesDblClick(Sender);
  end;
end;

procedure TFormVars.RBClearClick(Sender: TObject);
begin
  if RBValue.checked then EditInitialization.color:=clwindow
  else EditInitialization.color:=clsilver;
  EditInitialization.readonly:=not RBValue.checked;
  EditInitialization.tabstop:=RBValue.checked;
end;

procedure TFormVars.EditSetKeyPress(Sender: TObject; var Key: Char);
begin
  if key = #13 then begin
    key:=#0;
    BtnSet.click;
  end;
end;

procedure TFormVars.BtnSetClick(Sender: TObject);
begin
  if Datawell.TableVariables.fieldbyname('Variable').asstring = '' then begin
    messagedlg('There is no variable to set',mtinformation,[mbok],0);
    exit;
  end;
  Datawell.TableVariables.edit;
  Datawell.TableVariables.fieldbyname('Value').asstring:=EditSet.text;
  Datawell.TableVariables.post;
  Datawell.TableVariables.refresh;
  GridVariables.refresh;
  EditSet.text:='';
  BtnDone.setfocus;
end;

procedure TFormVars.BtnVarsClick(Sender: TObject);
var form: TFormVarGrab;
begin
  form:=TFormVarGrab.create(self);
  form.setup;
  form.BtnVars.enabled:=false;
  form.BtnOK1.enabled:=false;
  form.BtnOK2.enabled:=false;
  form.BtnOK3.enabled:=false;
  form.BtnOK4.enabled:=false;
  //Disable Evaluators
  form.ListEvaluators.enabled:=false;
  form.ListEvaluators.color:=clsilver;
  form.Label4.enabled:=false;
  form.Label4a.enabled:=false;
  form.showmodal;
  form.free;
end;

end.
